/*
* JBoss, Home of Professional Open Source.
* Copyright 2006, Red Hat Middleware LLC, and individual contributors
* as indicated by the @author tags. See the copyright.txt file in the
* distribution for a full listing of individual contributors. 
*
* This is free software; you can redistribute it and/or modify it
* under the terms of the GNU Lesser General Public License as
* published by the Free Software Foundation; either version 2.1 of
* the License, or (at your option) any later version.
*
* This software is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
* Lesser General Public License for more details.
*
* You should have received a copy of the GNU Lesser General Public
* License along with this software; if not, write to the Free
* Software Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
* 02110-1301 USA, or see the FSF site: http://www.fsf.org.
*/ 
package org.jboss.lang.reflect;

import java.util.Arrays;

/**
 * 
 * @author Paul Gier
 * @version $Revision: 1.1 $
 * 
 */
public class WildcardTypeImpl implements WildcardType
{

   private static final char WILDCARD = '*';
   private static final char WILDCARD_UPPER = '+';
   private static final char WILDCARD_LOWER = '-';
   
   private static final Object[] DEFAULT_UPPER_BOUND = new Object[]{Object.class};
   private static final Object[] DEFAULT_LOWER_BOUND = new Object[0];

   private Object bound;
   
   private char kind;
   
   /**
    * Create a new wildcard type
    * @param bound - The upper or lower bound depending on the kind
    * @param kind - Kind can be either '*' (wildcard), '+' (upper-bound), or '-' (lower-bound)
    */
   public WildcardTypeImpl(Object bound, char kind)
   {
      this.bound = bound;
      this.kind = kind;
   }
   
   public Object[] getLowerBounds()
   {
      if (kind == WILDCARD_LOWER)
      {
         return new Object[] {bound};
      }
      else 
      {
         return DEFAULT_LOWER_BOUND;
      }
   }

   public Object[] getUpperBounds()
   {
      if (kind == WILDCARD_UPPER)
      {
         return new Object[] {bound};
      }
      else 
      {
         return DEFAULT_UPPER_BOUND;
      }
   }
   
   public int hashCode() {
      return Arrays.hashCode(getUpperBounds()) ^ Arrays.hashCode(getLowerBounds());
  }

  public boolean equals(Object obj) {
      if (obj instanceof WildcardType) {
          if (obj == this) return true;
          WildcardType type = ((WildcardType) obj);
          return Arrays.equals(getUpperBounds(), type.getUpperBounds()) &&
                  Arrays.equals(getLowerBounds(), type.getLowerBounds());
      }
      return false;
  }

  public String toString()
   {
      if (bound == null)
      {
         return "?";
      }
      
      String boundName = "";
      if (bound instanceof Class)
      {
         boundName = ((Class)bound).getName();
      } 
      else 
      {
         boundName = bound.toString();
      }
      
      if (kind == WILDCARD_UPPER)
      {
         return ("? extends " + boundName);
      }
      else
      {
         return ("? super " + boundName);
      }
   }

}
